<?php

declare(strict_types=1);

namespace Drupal\slots\Plugin\Field\FieldWidget;

use Drupal\Core\Cache\Cache;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\conditions_field\Plugin\Field\FieldWidget\ConditionsGroupsWidget;

/**
 * Defines the 'slots' field widget.
 *
 * @FieldWidget(
 *   id = "slots",
 *   label = @Translation("Slots (EXPERIMENTAL)"),
 *   field_types = {"slots"},
 * )
 */
class SlotsWidget extends ConditionsGroupsWidget {

  /**
   * {@inheritdoc}
   */
  public function form(FieldItemListInterface $items, array &$form, FormStateInterface $form_state, $get_delta = NULL) {
    $field_widget_complete_form = parent::form($items, $form, $form_state, $get_delta);

    // Show condition widget only if this checkbox is checked.
    $field_widget_complete_form['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('This content shall be displayed in slots.'),
      '#default_value' => !$items->isEmpty(),
      '#weight' => -1,
    ];

    $parents = implode('][', $field_widget_complete_form['#parents']);
    $field_widget_complete_form['condition_groups']['#states'] = [
      'visible' => [':input[name="' . $parents . '[enable]"]' => ['checked' => TRUE]],
    ];

    return $field_widget_complete_form;
  }

  /**
   * {@inheritdoc}
   */
  public function extractFormValues(FieldItemListInterface $items, array $form, FormStateInterface $form_state) {
    $old_slot_ids = $this->extractSlotIds($items);
    $values = parent::extractFormValues($items, $form, $form_state);
    $new_sot_ids = $this->extractSlotIds($items);

    // Invalidate all cache tags of the form slot_id:ID.
    $cache_tags = array_map(function ($s) {
      return 'slot_id:' . $s;

    }, array_merge($old_slot_ids, $new_sot_ids));
    $cache_tags[] = 'slot_list';
    Cache::invalidateTags($cache_tags);

    return $values;
  }

  /**
   * Extract slot ids from items.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The field list items to extract the slot ids from.
   *
   * @return array
   *   The slot ids.
   */
  protected function extractSlotIds(FieldItemListInterface $items): array {
    $slot_ids = [];
    foreach ($items->getValue() as $condition_group) {
      if (empty($condition_group['value'])) {
        continue;
      }
      foreach (Json::decode($condition_group['value']) as $condition) {
        if ($condition['id'] != 'slot') {
          continue;
        }

        $slot_ids[] = $condition['slot_id'];
      }
    }

    return $slot_ids;
  }

}
